
import os, sys
from datetime import datetime

from ketchup_src.ketchup_1_5_2.tags import get_spread
from ketchup_src.ketchup_1_5_2.spread import KETCHUP_TAGS, FRONTLINE_TAGS, Spread, InlineSpread


VERSION = '1.5.2'

STYLESHEET = '/Users/charleskoch/Documents/prgrm/Corndog/ketchup_src/ketchup_'+VERSION.replace('.', '_')+'/style.css'


########  BASE CLASSES  ########
################################

class Document():

	def __init__(self, filepath, page_name = None, output_name = None, output_dir = None, css_file = STYLESHEET):
		self.page_name = page_name
		self.filepath = filepath
		self.output_name = output_name
		self.output_dir = output_dir
		self.css_file = css_file
		self.document = []


	def add_text(self, text):
		myfile = text.split('\n')
		parse_text(myfile)


	def parse_text(self, myfile):
		prev_line = None
		spread = None
		for line in myfile:
			unstripped_line = line
			line = line.strip()

			if prev_line == '@page' and not spread:
				self.page_name = line
				pass
			elif prev_line == '@file' and not spread:
				self.output_name = line
				pass
			elif prev_line == '@outdir' and not spread:
				self.output_dir = line
				pass

			if line in KETCHUP_TAGS and (not spread or ( spread and spread.type not in ['@codeblock', '@literal', '@class', '@method'])):
				#print(line)
				if spread and not spread.is_empty():
					#print('closing ' + str(spread))
					self.document.append(spread)
					spread = None
				if line not in ['@page', '@file', '@outdir']:
					spread = get_spread(line)
					#print('opening ' + str(spread))
				if spread and spread.type in ['@hr','@br']:
					self.document.append(spread)
					#print('closing ' + str(spread))
					spread = None

			elif line == '' and (not spread or ( spread and spread.type not in ['@codeblock', '@literal'])):
				if spread and not spread.is_empty():
					#print('closing ' + str(spread))
					self.document.append(spread)
				spread = None

			elif line in ['@endcodeblock', '@endliteral']:
				if spread and not spread.is_empty():
					#print('closing ' + str(spread))
					self.document.append(spread)
				spread = None

			elif spread and spread.type in ['@codeblock', '@literal']:
				spread.add_text(unstripped_line)

			elif len(line) > 1 and line[0] in ['-', '*', '#'] and line[1] in ['-', '*', '#', ' '] and (not spread or spread.type not in ['@non-bulleted', '@bulleted', '@numbered', '@table', '@class', '@method']):
				if spread and not spread.is_empty():
					#print('closing ' + str(spread))
					self.document.append(spread)
				tag = FRONTLINE_TAGS[line[0]]
				spread = get_spread(tag)
				spread.add_text(line)
				#print('opening ' + str(spread))

			elif len(line) > 2 and line[:3] == '___':
				if spread and not spread.is_empty():
					#print('closing ' + str(spread))
					self.document.append(spread)
				spread = get_spread('@hr')
				#print('opening ' + str(spread))
				self.document.append(spread)
				#print('closing ' + str(spread))
				spread = None

			elif spread:
				spread.add_text(line)
			else:
				if prev_line not in ['@page', '@file', '@outdir']:
					spread = get_spread()
					spread.add_text(line)
					#print('opening ' + str(spread))
			prev_line = line

		if spread and not spread.is_empty():
			#print('closing ' + str(spread))
			self.document.append(spread)
			spread = None


	def render(self, index):
		result = ''
		for i in range(len(self.document)):
			spread = self.document[i]
			result += spread.render(str(i)) + '\n'
		return result




class Ketchup(Document):

	def begin(self):
		self.initialize_data()
		fullpath = os.path.abspath(self.filepath)
		self.parse_file(fullpath)
		filename, file_ext = os.path.splitext(self.filepath)
		filename = filename.replace('/', '.')
		filename = filename.replace('\\', '.')
		result = self.render(self.page_name)
		self.send_to_output(result, self.output_name, self.output_dir)


	def initialize_data(self):
		self.document = []
		filename, file_ext = os.path.splitext(self.filepath)
		filename = filename.replace('/', '.')
		filename = filename.replace('\\', '.')
		if self.page_name is None:
			self.page_name = filename
		if self.output_name is None:
			self.output_name = filename
		if self.output_dir is None:
			self.output_dir = os.getcwd()

		# if output_dir is not an absolute path, we need to make it one
		if not os.path.isabs(self.output_dir):
			self.output_dir = os.path.abspath(self.output_dir)


	def parse_file(self, filepath):
		with open(filepath, 'r') as myfile:
			self.parse_text(myfile)


	def minify_css(self, css_file):
		with open(css_file, 'r') as mycss:
			css = mycss.read()
			css = css.strip()
			css = css.replace('\t', '')
		return css


	def render_nav(self):
		result = '<div class="k-navbar">'
		for i in range(len(self.document)):
			spread = self.document[i]
			# if spread.type == '@imports':
			# 	result += '<a class="k-a" href="#' + str(i) + '"><div class="k-nav k-nav-import">Imports</div></a>\n'
			# if spread.type == '@params':
			# 	result += '<a class="k-a" href="#' + str(i) + '"><div class="k-nav k-nav-params">Parameters</div></a>\n'
			# if spread.type == '@attributes':
			# 	result += '<a class="k-a" href="#' + str(i) + '"><div class="k-nav k-nav-attributes">Attributes</div></a>\n'
			# if spread.type == '@constants':
			# 	result += '<a class="k-a" href="#' + str(i) + '"><div class="k-nav k-nav-constants">Constants</div></a>\n'
			# if spread.type == '@exceptions':
			# 	result += '<a class="k-a" href="#' + str(i) + '"><div class="k-nav k-nav-exceptions">Exceptions</div></a>\n'
			if spread.type in ['@h1','@h2','@h3','@h4']:
				num = spread.type[-1]
				result += '<a class="k-a" href="#' + str(i) + '"><div class="k-nav k-nav-header k-nh' + num + '">' + spread.text + '</div></a>\n'
			if spread.type == '@class':
				result += '<a class="k-a" href="#' + str(i) + '"><div class="k-nav k-nav-class">' + spread.text + '</div></a>\n'
			if spread.type == '@method':
				result += '<a class="k-a" href="#' + str(i) + '"><div class="k-nav k-nav-method">' + spread.text + '</div></a>\n'
		result += '</div>\n'
		return result


	def render(self, filename):
		result = '<html><head>\n'
		title = filename.split('.')[-1]
		result += '<title>' + title + '</title>\n'
		result += '<style>' + self.minify_css(self.css_file) + '</style>\n'
		result += '</head><body>\n'
		result += '<a class="k-a" href=""><div class="k-page-title">' + filename + '</div></a>\n'
		result += '<div class="k-page">\n'
		result += self.render_nav()
		result += '<div class="k-content">\n'
		for i in range(len(self.document)):
			spread = self.document[i]
			result += spread.render(str(i)) + '\n'
		right_now = datetime.now()
		result += '<div class="k-spacer">Generated ' + right_now.strftime('%I:%M%p  %d %b %Y') + '</br>'
		result += 'Documentation generated by Ketchup v'+ VERSION +'</br>'
		result += 'Developed by Charles Koch - 2017</div>'
		result += '</div></div></body></html>'
		return result


	def send_to_output(self, text, output_name, output_dir):
		file_path = os.path.join(output_dir, output_name + '.html')
		with open(file_path, 'w') as outfile:
			outfile.write(text)

