
KETCHUP_TAGS = ['@byline',		# multiline
				'@imports',		# multiline
				'@class',		# single line
				'@method',		# single line
				'@attributes',	# multiline list
				'@constants',	# multiline list
				'@params',		# multiline list
				'@return',		# single line
				'@exceptions',	# multiline list
				'@deflist',		# multiline list
				'@h1','@h2',	# single line
				'@h3','@h4',	# single line
				'@m1','@m2',	# single line
				'@m3','@m4',	# single line
				'@table',		# multiline
				'@hr',			# horizontal rule
				'@br',			# break line
				'@codeblock',	# multiline
				'@endcodeblock',# end tag
				'@literal',		# multiline
				'@endliteral',	# end tag
				'@note',		# single line
				'@page',		# single line
				'@file',		# single line
				'@outdir',		# single line
				]

FRONTLINE_TAGS = {
				'-': '@non-bulleted',
				'*': '@bulleted',
				'#': '@numbered'
				}
				

########  BASE SPREAD  ########
###############################

class Spread():

	def __init__(self, tag = None):
		self.type = tag
		self.text = ''

	def __str__(self):
		return str(self.type)

	def is_empty(self):
		return self.text == ''

	def add_text(self, text):
		text = text.replace('<', '&lt;')
		text = text.replace('>', '&gt;')
		self.text += ' ' + text

	def render_text(self, text):
		text = text.replace('<', '&lt;')
		text = text.replace('>', '&gt;')
		return text

	def render(self, index):
		result = ''
		result += '<div id="' + index + '" class="k-paragraph">' + self.render_text(self.text) + '</div>'
		return result


########  INLINE SPREAD  ########
#################################

class InlineSpread(Spread):

	def render_text(self, text):
		text = text.replace('<', '&lt;')
		text = text.replace('>', '&gt;')
		return self.render_inline(text)


	def render_inline(self, text):
		result = ''
		disregard_next = False
		codeline_start = False
		codeline_end = False
		link_text = ''
		tag = None
		bold = False
		italic = False
		for i in range(len(text)):
			char = text[i]
			if char == '\\':
				disregard_next = True
			elif disregard_next:
				disregard_next = False
				result += char
			else:
				if tag == 'codeline':
					if (char == '-' and (i < (len(text)-1) and text[i+1] == '/')) or (codeline_end and char == '/'):
						codeline_end = True
					else:
						codeline_end = False
					if char == '/' and codeline_end:
						result += '</span>'
						codeline_start = False
						codeline_end = False
						tag = None
					elif char == '-' and (codeline_end or codeline_start):
						pass
					else:
						codeline_start = False
						result += char
						codeline_end = False
				elif char == '/' and (i == 0 or text[i-1] == ' ') and tag not in ['code', 'codeline']:
					if i < (len(text)-1) and text[i+1] == '-':
						tag = 'codeline'
						codeline_start = True
					else:
						tag = 'code'
					result += '<span class="k-code">'
				elif char == ' ' and tag == 'code':
					result += '</span> '
					tag = None
				elif char == '!' and (i == 0 or text[i-1] == ' ') and tag not in ['link']:
					tag = 'link'
					link_text = ''
					result += '<a class="k-link" target="_blank" href="'
				elif char == ' ' and tag == 'link':
					result += '">' + link_text + '</a> '
					tag = None
				elif char == '*' and tag is None:
					if (i < (len(text)-1) and text[i+1] != ' ') and not bold:
						bold = True
						result += '<strong>'
					elif bold:
						bold = False
						result += '</strong>'
				elif char == '_' and tag is None:
					if (i < (len(text)-1) and text[i+1] != ' ') and not italic:
						italic = True
						result += '<em>'
					elif italic:
						italic = False
						result += '</em>'
				elif char =='@' and tag is None:
					result += '<br />'
				else:
					if tag == 'link':
						link_text += char
					result += char
		if tag in ['code','codeline']:
			result += '</span> '
		elif tag in ['link']:
			result += '">' + link_text + '</a> '
		if bold:
			result += '</strong>'
		if italic:
			result += '</em>'

		return result





